@@ -1,17 +1,37 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
+
+
 #include <string.h>
 #include <stdlib.h>
+#include <stdio.h>
+
 #include <iostream>
 #include <sstream>
+#include <boost/variant.hpp>
+#include <boost/scoped_array.hpp>
+
 #include <cppconn/exception.h>
 #include "mysql_util.h"
 #include "mysql_connection.h"
@@ -23,8 +43,8 @@
 #include "mysql_warning.h"
 #include "mysql_resultbind.h"
 
+#include "nativeapi/native_statement_wrapper.h"
 
-#define mysql_stmt_conn(s) (s)->mysql
 
 #include "mysql_debug.h"
 
@@ -35,22 +55,178 @@ namespace sql
 namespace mysql
 {
 
+static const unsigned int MAX_SEND_LONGDATA_BUFFER= 1 << 18; //1<<18=256k (for istream)
+static const unsigned int MAX_SEND_LONGDATA_CHUNK=  1 << 18; //1<<19=512k (for string)
+
+// Visitor class to send long data contained in blob_bind
+class LongDataSender : public boost::static_visitor<bool>
+{
+	unsigned	position;
+	boost::shared_ptr< NativeAPI::NativeStatementWrapper > proxy;
+	boost::shared_ptr<MySQL_DebugLogger> logger;
+
+	LongDataSender()
+	{}
+
+public:
+
+	LongDataSender(unsigned int i, boost::shared_ptr< NativeAPI::NativeStatementWrapper > & _proxy,
+    boost::shared_ptr<MySQL_DebugLogger> _logger)
+		: position	( i			)
+		, proxy		( _proxy	)
+		, logger	( _logger	)
+	{
+	}
+
+	bool operator()(std::istream * my_blob) const
+	{
+		CPP_ENTER("LongDataSender::operator()(std::istream *)");
+		if (my_blob == NULL)
+			return false;
+
+		//char buf[MAX_SEND_LONGDATA_BUFFER];
+		boost::scoped_array<char> buf(new char[MAX_SEND_LONGDATA_BUFFER]);
+
+		do {
+			if (my_blob->eof()) {
+				break;
+			}
+			my_blob->read(buf.get(), MAX_SEND_LONGDATA_BUFFER);
+
+			if (my_blob->bad()) {
+				throw SQLException("Error while reading from blob (bad)");
+			} else if (my_blob->fail()) {
+				if (!my_blob->eof()) {
+					throw SQLException("Error while reading from blob (fail)");
+				}
+			}
+			if (proxy->send_long_data(position, buf.get(), static_cast<unsigned long>(my_blob->gcount()))) {
+				CPP_ERR_FMT("Couldn't send long data : %d:(%s) %s", proxy->errNo(), proxy->sqlstate().c_str(), proxy->error().c_str());
+				switch (proxy->errNo()) {
+				case CR_OUT_OF_MEMORY:
+					throw std::bad_alloc();
+				case CR_INVALID_BUFFER_USE:
+					throw InvalidArgumentException("MySQL_Prepared_Statement::setBlob: can't set blob value on that column");
+				case CR_SERVER_GONE_ERROR:
+				case CR_COMMANDS_OUT_OF_SYNC:
+				default:
+					sql::mysql::util::throwSQLException(*proxy.get());
+				}
+			}
+		} while (1);
+
+		return true;
+	}
+
+	bool operator()(sql::SQLString * str) const
+	{
+		CPP_ENTER("LongDataSender::operator()(sql::SQLString *)");
+		if ( str == NULL )
+			return false;
+
+		unsigned int sent= 0, chunkSize;
+
+		while (sent < str->length())
+		{
+			chunkSize= (sent + MAX_SEND_LONGDATA_CHUNK > str->length()
+						? str->length() - sent
+						: MAX_SEND_LONGDATA_CHUNK);
+
+			if (proxy->send_long_data(position, str->c_str() + sent, chunkSize)) {
+				CPP_ERR_FMT("Couldn't send long data : %d:(%s) %s", proxy->errNo(), proxy->sqlstate().c_str(), proxy->error().c_str());
+				switch (proxy->errNo()) {
+				case CR_OUT_OF_MEMORY:
+					throw std::bad_alloc();
+				case CR_INVALID_BUFFER_USE:
+					throw InvalidArgumentException("MySQL_Prepared_Statement::setBlob: can't set blob value on that column");
+				case CR_SERVER_GONE_ERROR:
+				case CR_COMMANDS_OUT_OF_SYNC:
+				default:
+					sql::mysql::util::throwSQLException(*proxy.get());
+				}
+			}
+
+			sent+= chunkSize;
+		}
+
+		return true;
+	}
+};
+
+
+class BlobBindDeleter : public boost::static_visitor<>
+{
+public:
+
+	void operator()(sql::SQLString *& str) const
+	{
+		if (str != NULL) {
+		delete str;
+		str= NULL;
+		}
+	}
+
+	void operator()(std::istream *& my_blob) const
+	{
+		if (my_blob!= NULL) {
+			delete my_blob;
+			my_blob= NULL;
+		}
+	}
+};
+
+class BlobIsNull : public boost::static_visitor<bool>
+{
+public:
+
+	bool operator()(sql::SQLString *& str) const
+	{
+		return str == NULL;
+	}
+
+	bool operator()(std::istream *& my_blob) const
+	{
+		return my_blob == NULL;
+	}
+};
+
+
+void resetBlobBind(MYSQL_BIND & param)
+{
+	delete [] static_cast<char *>(param.buffer);
+
+	param.buffer_type=		MYSQL_TYPE_LONG_BLOB;
+	param.buffer=			NULL;
+	param.buffer_length=	0;
+	param.is_null_value=	0;
+
+	delete param.length;
+	param.length=			new unsigned long(0);
+}
 
 
 class MySQL_ParamBind
 {
-	unsigned int param_count;
-	sql::mysql::util::my_array_guard< MYSQL_BIND > bind;
-	sql::mysql::util::my_array_guard< bool > value_set;
-	sql::mysql::util::my_array_guard< bool > delete_blob_after_execute;
+public:
 
-	sql::mysql::util::my_array_guard< std::istream	* > blob_bind;
+	typedef boost::variant< std::istream *, sql::SQLString *> Blob_t;
+
+private:
+
+	unsigned int param_count;
+	boost::scoped_array< MYSQL_BIND > bind;
+	boost::scoped_array< bool > value_set;
+	boost::scoped_array< bool > delete_blob_after_execute;
+
+	typedef std::map<unsigned int, Blob_t > Blobs;
+
+	Blobs blob_bind;
 
 public:
 
 	MySQL_ParamBind(unsigned int paramCount)
 		: param_count(paramCount), bind(NULL), value_set(NULL),
-		  delete_blob_after_execute(NULL), blob_bind(NULL)
+		  delete_blob_after_execute(NULL)
 	{
 		if (param_count) {
 			bind.reset(new MYSQL_BIND[paramCount]);
@@ -63,9 +239,6 @@ public:
 				value_set[i] = false;
 				delete_blob_after_execute[i] = false;
 			}
-
-			blob_bind.reset(new std::istream *[paramCount]);
-			memset(blob_bind.get(), 0, sizeof(std::istream *) * paramCount);
 		}
 	}
 
@@ -73,13 +246,11 @@ public:
 	{
 		clearParameters();
 
-		if (blob_bind.get()) {
-			for (unsigned int i = 0; i < param_count; ++i) {
-				if (delete_blob_after_execute[i]) {
-					delete_blob_after_execute[i] = false;
-					delete blob_bind[i];
-					blob_bind[i] = NULL;
-				}
+		for (Blobs::iterator it= blob_bind.begin();
+			it != blob_bind.end(); ++it) {
+			if (delete_blob_after_execute[it->first]) {
+				delete_blob_after_execute[it->first] = false;
+				boost::apply_visitor(::sql::mysql::BlobBindDeleter(), it->second);
 			}
 		}
 	}
@@ -94,20 +265,38 @@ public:
 		value_set[position] = false;
 		if (delete_blob_after_execute[position]) {
 			delete_blob_after_execute[position] = false;
-			delete blob_bind[position];
-			blob_bind[position] = NULL;
+			boost::apply_visitor(::sql::mysql::BlobBindDeleter(),blob_bind[position]);
+			blob_bind.erase(position);
 		}
 	}
 
-	void setBlob(unsigned int position, std::istream * blob, bool delete_after_execute)
+
+	void setBlob(unsigned int position, Blob_t & blob, bool delete_after_execute)
 	{
-		if (blob_bind[position] && delete_blob_after_execute[position]) {
-			delete blob_bind[position];
+		set(position);
+
+		resetBlobBind(bind[position]);
+
+		Blobs::iterator it = blob_bind.find(position);
+		if (it != blob_bind.end() && delete_blob_after_execute[position]) {
+				boost::apply_visitor(::sql::mysql::BlobBindDeleter(), it->second);
+			}
+
+		if (boost::apply_visitor(::sql::mysql::BlobIsNull(), blob))
+		{
+		  if (it != blob_bind.end())
+			blob_bind.erase(it);
+
+		  delete_blob_after_execute[position] = false;
+		}
+		else
+		{
+			  blob_bind[position] = blob;
+			  delete_blob_after_execute[position] = delete_after_execute;
 		}
-		blob_bind[position] = blob;
-		delete_blob_after_execute[position] = delete_after_execute;
 	}
 
+
 	bool isAllSet()
 	{
 		for (unsigned int i = 0; i < param_count; ++i) {
@@ -118,6 +307,7 @@ public:
 		return true;
 	}
 
+
 	void clearParameters()
 	{
 		for (unsigned int i = 0; i < param_count; ++i) {
@@ -126,44 +316,55 @@ public:
 			delete[] (char*) bind[i].buffer;
 			bind[i].buffer = NULL;
 			if (value_set[i]) {
-				if (blob_bind[i] && delete_blob_after_execute[i]) {
-					delete blob_bind[i];
+				Blobs::iterator it= blob_bind.find(i);
+				if (it != blob_bind.end() && delete_blob_after_execute[i]) {
+					boost::apply_visitor(::sql::mysql::BlobBindDeleter(), it->second);
+					blob_bind.erase(it);
 				}
-				blob_bind[i] = NULL;
+				blob_bind[i] = Blob_t();
 				value_set[i] = false;
 			}
 		}
 	}
 
-	MYSQL_BIND * get()
+
+	// Name get() was too confusing, since class objects are used with smart pointers
+	MYSQL_BIND * getBindObject()
 	{
 		return bind.get();
 	}
 
-	std::istream * getBlobObject(unsigned int position)
+
+	boost::variant< std::istream *, SQLString *> getBlobObject(unsigned int position)
 	{
-		return blob_bind[position];
+		Blobs::iterator it= blob_bind.find( position );
+
+		if (it != blob_bind.end())
+			return it->second;
+
+		return Blob_t();
 	}
 
 };
 
 
-
 /* {{{ MySQL_Prepared_Statement::MySQL_Prepared_Statement() -I- */
 MySQL_Prepared_Statement::MySQL_Prepared_Statement(
-			MYSQL_STMT *s, sql::Connection * conn, sql::ResultSet::enum_type rset_type,
-			sql::mysql::util::my_shared_ptr< MySQL_DebugLogger > * log
+			boost::shared_ptr< NativeAPI::NativeStatementWrapper > & s, sql::Connection * conn,
+			sql::ResultSet::enum_type rset_type, boost::shared_ptr< MySQL_DebugLogger > & log
 		)
-	:connection(conn), stmt(s), isClosed(false),
-	 logger(log? log->getReference():NULL), resultset_type(rset_type)
+	:connection(conn), proxy(s), isClosed(false), warningsHaveBeenLoaded(true), logger(log),
+		resultset_type(rset_type), result_bind(new MySQL_ResultBind(proxy, logger)),
+		warningsCount(0)
+
 {
 	CPP_ENTER("MySQL_Prepared_Statement::MySQL_Prepared_Statement");
 	CPP_INFO_FMT("this=%p", this);
-	param_count = mysql_stmt_param_count(s);
+	param_count = proxy->param_count();
 	param_bind.reset(new MySQL_ParamBind(param_count));
 
-	res_meta.reset(new MySQL_Prepared_ResultSetMetaData(stmt, logger));
-	param_meta.reset(new MySQL_ParameterMetaData(stmt));
+	res_meta.reset(new MySQL_PreparedResultSetMetaData(proxy, logger));
+	param_meta.reset(new MySQL_ParameterMetaData(proxy));
 }
 /* }}} */
 
@@ -171,18 +372,14 @@ MySQL_Prepared_Statement::MySQL_Prepared_Statement(
 /* {{{ MySQL_Prepared_Statement::~MySQL_Prepared_Statement() -I- */
 MySQL_Prepared_Statement::~MySQL_Prepared_Statement()
 {
-	/* Don't remove the block or we can get into problems with logger */
-	{
-		CPP_ENTER("MySQL_Prepared_Statement::~MySQL_Prepared_Statement");
-		/*
-		  This will free param_bind.
-		  We should not do it or there will be double free.
-		*/
-		if (!isClosed) {
-			closeIntern();
-		}
+	CPP_ENTER("MySQL_Prepared_Statement::~MySQL_Prepared_Statement");
+	/*
+	  This will free param_bind.
+	  We should not do it or there will be double free.
+	*/
+	if (!isClosed) {
+		closeIntern();
 	}
-	logger->freeReference();
 }
 /* }}} */
 
@@ -192,39 +389,16 @@ bool
 MySQL_Prepared_Statement::sendLongDataBeforeParamBind()
 {
 	CPP_ENTER("MySQL_Prepared_Statement::sendLongDataBeforeParamBind");
-	MYSQL_BIND * bind = param_bind->get();
-	char buf[1024];
+
+	MYSQL_BIND * bind= param_bind->getBindObject();
+
 	for (unsigned int i = 0; i < param_count; ++i) {
 		if (bind[i].buffer_type == MYSQL_TYPE_LONG_BLOB) {
-			std::istream * my_blob = param_bind->getBlobObject(i);
-			do {
-				if ((my_blob->rdstate() & std::istream::eofbit) != 0 ) {
-					break;
-				}
-				my_blob->read(buf, sizeof(buf));
-
-				if ((my_blob->rdstate() & std::istream::badbit) != 0) {
-					throw SQLException("Error while reading from blob (bad)");
-				} else if ((my_blob->rdstate() & std::istream::failbit) != 0) {
-					if ((my_blob->rdstate() & std::istream::eofbit) == 0) {
-						throw SQLException("Error while reading from blob (fail)");
-					}
-				}
-				if (mysql_stmt_send_long_data(stmt, i, buf, static_cast<unsigned long>(my_blob->gcount()))) {
-					CPP_ERR_FMT("Couldn't send long data : %d:(%s) %s", mysql_stmt_errno(stmt), mysql_stmt_sqlstate(stmt), mysql_stmt_error(stmt));
-					switch (mysql_stmt_errno(stmt)) {
-						case CR_OUT_OF_MEMORY:
-							throw std::bad_alloc();
-						case CR_INVALID_BUFFER_USE:
-							throw InvalidArgumentException("MySQL_Prepared_Statement::setBlob: can't set blob value on that column");
-						case CR_SERVER_GONE_ERROR:
-						case CR_COMMANDS_OUT_OF_SYNC:
-						default:		
-							sql::mysql::util::throwSQLException(stmt);
-					}
-				}
-			} while (1);
+			::sql::mysql::LongDataSender lv(i, proxy, logger);
+      MySQL_ParamBind::Blob_t dummy(param_bind->getBlobObject(i));
+			boost::apply_visitor(lv, dummy);
 		}
+
 	}
 	return true;
 }
@@ -240,14 +414,20 @@ MySQL_Prepared_Statement::do_query()
 		CPP_ERR("Value not set for all parameters");
 		throw sql::SQLException("Value not set for all parameters");
 	}
-	if (mysql_stmt_bind_param(stmt, param_bind->get())) {
-		CPP_ERR_FMT("Couldn't bind : %d:(%s) %s", mysql_stmt_errno(stmt), mysql_stmt_sqlstate(stmt), mysql_stmt_error(stmt));
-		sql::mysql::util::throwSQLException(stmt);
+
+	if (proxy->bind_param(param_bind->getBindObject())) {
+		CPP_ERR_FMT("Couldn't bind : %d:(%s) %s", proxy->errNo(), proxy->sqlstate().c_str(), proxy->error().c_str());
+		sql::mysql::util::throwSQLException(*proxy.get());
 	}
-	if (!sendLongDataBeforeParamBind() || mysql_stmt_execute(stmt)) {
-		CPP_ERR_FMT("Couldn't execute : %d:(%s) %s", mysql_stmt_errno(stmt), mysql_stmt_sqlstate(stmt), mysql_stmt_error(stmt));
-		sql::mysql::util::throwSQLException(stmt);
+
+	if (!sendLongDataBeforeParamBind() || proxy->execute()) {
+		CPP_ERR_FMT("Couldn't execute : %d:(%s) %s", proxy->errNo(), proxy->sqlstate().c_str(), proxy->error().c_str());
+		sql::mysql::util::throwSQLException(*proxy.get());
 	}
+
+	warningsCount= proxy->warning_count();
+
+	warningsHaveBeenLoaded= false;
 }
 /* }}} */
 
@@ -284,16 +464,16 @@ MySQL_Prepared_Statement::execute()
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
 	do_query();
-	return (mysql_stmt_field_count(stmt) > 0);
+	return (proxy->field_count() > 0);
 }
 /* }}} */
 
 
 /* {{{ MySQL_Prepared_Statement::execute() -U- */
 bool
-MySQL_Prepared_Statement::execute(const std::string&)
+MySQL_Prepared_Statement::execute(const sql::SQLString&)
 {
-	CPP_ENTER("MySQL_Prepared_Statement::execute(const std::string& sql)");
+	CPP_ENTER("MySQL_Prepared_Statement::execute(const sql::SQLString& sql)");
 	throw sql::MethodNotImplementedException("MySQL_Prepared_Statement::execute");
 	return false; // fool compilers
 }
@@ -311,21 +491,19 @@ MySQL_Prepared_Statement::executeQuery()
 	do_query();
 
 	my_bool	bool_tmp=1;
-	mysql_stmt_attr_set(stmt, STMT_ATTR_UPDATE_MAX_LENGTH, &bool_tmp);
+	proxy->attr_set( STMT_ATTR_UPDATE_MAX_LENGTH, &bool_tmp);
 	sql::ResultSet::enum_type tmp_type;
 	if (resultset_type == sql::ResultSet::TYPE_SCROLL_INSENSITIVE) {
-		mysql_stmt_store_result(stmt);
+		if (proxy->store_result()) {
+			sql::mysql::util::throwSQLException(*proxy.get());
+		}
 		tmp_type = sql::ResultSet::TYPE_SCROLL_INSENSITIVE;
 	} else if (resultset_type == sql::ResultSet::TYPE_FORWARD_ONLY) {
 		tmp_type = sql::ResultSet::TYPE_FORWARD_ONLY;
 	} else {
 		throw SQLException("Invalid value for result set type");
 	}
-	
-	std::auto_ptr< MySQL_ResultBind > result_bind(new MySQL_ResultBind(stmt, logger));
-	
-	sql::ResultSet * tmp = new MySQL_Prepared_ResultSet(stmt, result_bind.get(), tmp_type, this, logger);
-	result_bind.release();
+	sql::ResultSet * tmp = new MySQL_Prepared_ResultSet(proxy, result_bind, tmp_type, this, logger);
 
 	CPP_INFO_FMT("rset=%p", tmp);
 	return tmp;
@@ -335,7 +513,7 @@ MySQL_Prepared_Statement::executeQuery()
 
 /* {{{ MySQL_Prepared_Statement::executeQuery() -U- */
 sql::ResultSet *
-MySQL_Prepared_Statement::executeQuery(const std::string&)
+MySQL_Prepared_Statement::executeQuery(const sql::SQLString&)
 {
 	throw sql::MethodNotImplementedException("MySQL_Prepared_Statement::executeQuery"); /* TODO - what to do? Comes from Statement */
 	return NULL; // fool compilers
@@ -351,14 +529,14 @@ MySQL_Prepared_Statement::executeUpdate()
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
 	do_query();
-	return static_cast<int>(mysql_stmt_affected_rows(stmt));
+	return static_cast<int>(proxy->affected_rows());
 }
 /* }}} */
 
 
 /* {{{ MySQL_Prepared_Statement::executeUpdate() -U- */
 int
-MySQL_Prepared_Statement::executeUpdate(const std::string&)
+MySQL_Prepared_Statement::executeUpdate(const sql::SQLString&)
 {
 	throw sql::MethodNotImplementedException("MySQL_Prepared_Statement::executeUpdate"); /* TODO - what to do? Comes from Statement */
 	return 0; // fool compilers
@@ -368,7 +546,7 @@ MySQL_Prepared_Statement::executeUpdate(const std::string&)
 
 /* {{{ MySQL_Prepared_Statement::setBigInt() -I- */
 void
-MySQL_Prepared_Statement::setBigInt(unsigned int parameterIndex, const std::string& value)
+MySQL_Prepared_Statement::setBigInt(unsigned int parameterIndex, const sql::SQLString& value)
 {
 	CPP_ENTER("MySQL_Prepared_Statement::setBigInt");
 	setString(parameterIndex, value);
@@ -377,17 +555,20 @@ MySQL_Prepared_Statement::setBigInt(unsigned int parameterIndex, const std::stri
 
 
 /* {{{ MySQL_Prepared_Statement::setBlob_intern() -I- */
+/*
 void
-MySQL_Prepared_Statement::setBlob_intern(unsigned int parameterIndex, std::istream * blob, bool deleteBlobAfterExecute)
+setBlob_intern(unsigned int parameterIndex
+										 , / *boost::variant< std::istream *, sql::SQLString *>* /MySQL_ParamBind::Blob_t & blob
+                                         , bool deleteBlobAfterExecute)
 {
 	CPP_ENTER("MySQL_Prepared_Statement::setBlob_intern");
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
 
-	--parameterIndex; /* DBC counts from 1 */
+	--parameterIndex; / * DBC counts from 1 * /
 
 	param_bind->set(parameterIndex);
-	MYSQL_BIND * param = &param_bind->get()[parameterIndex];
+	MYSQL_BIND * param = &param_bind->getBindObject()[parameterIndex];
 
 	delete [] static_cast<char *>(param->buffer);
 
@@ -400,7 +581,8 @@ MySQL_Prepared_Statement::setBlob_intern(unsigned int parameterIndex, std::istre
 	param->length = new unsigned long(0);
 
 	param_bind->setBlob(parameterIndex, blob, deleteBlobAfterExecute);
-}
+}*/
+
 /* }}} */
 
 
@@ -416,7 +598,8 @@ MySQL_Prepared_Statement::setBlob(unsigned int parameterIndex, std::istream * bl
 		throw InvalidArgumentException("MySQL_Prepared_Statement::setBlob: invalid 'parameterIndex'");
 	}
 
-	setBlob_intern(parameterIndex, blob, false);
+	MySQL_ParamBind::Blob_t dummy(blob);
+	param_bind->setBlob(--parameterIndex, dummy, false);
 }
 /* }}} */
 
@@ -433,7 +616,7 @@ MySQL_Prepared_Statement::setBoolean(unsigned int parameterIndex, bool value)
 
 /* {{{ MySQL_Prepared_Statement::setDateTime() -I- */
 void
-MySQL_Prepared_Statement::setDateTime(unsigned int parameterIndex, const std::string& value)
+MySQL_Prepared_Statement::setDateTime(unsigned int parameterIndex, const sql::SQLString& value)
 {
 	CPP_ENTER("MySQL_Prepared_Statement::setDateTime");
 	setString(parameterIndex, value);
@@ -470,7 +653,7 @@ allocate_buffer_for_type(enum_field_types t)
 		case MYSQL_TYPE_VAR_STRING:
 #endif
 		case MYSQL_TYPE_STRING:
-			return BufferSizePair(NULL, 0);
+			return BufferSizePair(NULLCSTR, 0);
 
 #if A1
 		// We don't use these now. When we have setXXX, we can enable them
@@ -492,7 +675,7 @@ allocate_buffer_for_type(enum_field_types t)
 		case MYSQL_TYPE_BIT:
 #endif
 		case MYSQL_TYPE_NULL:
-			return BufferSizePair(NULL, 0);
+			return BufferSizePair(NULLCSTR, 0);
 		default:
 			throw sql::InvalidArgumentException("allocate_buffer_for_type: invalid result_bind data type");
 	}
@@ -512,8 +695,9 @@ MySQL_Prepared_Statement::setDouble(unsigned int parameterIndex, double value)
 	}
 	--parameterIndex; /* DBC counts from 1 */
 
-	if (param_bind->getBlobObject(parameterIndex)) {
-		param_bind->setBlob(parameterIndex, NULL, false);
+	{
+		MySQL_ParamBind::Blob_t dummy;
+    param_bind->setBlob(parameterIndex, dummy, false);
 		param_bind->unset(parameterIndex);
 	}
 
@@ -522,7 +706,7 @@ MySQL_Prepared_Statement::setDouble(unsigned int parameterIndex, double value)
 	BufferSizePair p = allocate_buffer_for_type(t);
 
 	param_bind->set(parameterIndex);
-	MYSQL_BIND * param = &param_bind->get()[parameterIndex];
+	MYSQL_BIND * param = &param_bind->getBindObject()[parameterIndex];
 
 	param->buffer_type	= t;
 	delete [] static_cast<char *>(param->buffer);
@@ -551,8 +735,9 @@ MySQL_Prepared_Statement::setInt(unsigned int parameterIndex, int32_t value)
 	}
 	--parameterIndex; /* DBC counts from 1 */
 
-	if (param_bind->getBlobObject(parameterIndex)) {
-		param_bind->setBlob(parameterIndex, NULL, false);
+	{
+		MySQL_ParamBind::Blob_t dummy;
+		param_bind->setBlob(parameterIndex, dummy, false);
 		param_bind->unset(parameterIndex);
 	}
 
@@ -561,7 +746,7 @@ MySQL_Prepared_Statement::setInt(unsigned int parameterIndex, int32_t value)
 	BufferSizePair p = allocate_buffer_for_type(t);
 
 	param_bind->set(parameterIndex);
-	MYSQL_BIND * param = &param_bind->get()[parameterIndex];
+	MYSQL_BIND * param = &param_bind->getBindObject()[parameterIndex];
 
 	param->buffer_type	= t;
 	delete [] static_cast<char *>(param->buffer);
@@ -590,8 +775,9 @@ MySQL_Prepared_Statement::setUInt(unsigned int parameterIndex, uint32_t value)
 	}
 	--parameterIndex; /* DBC counts from 1 */
 
-	if (param_bind->getBlobObject(parameterIndex)) {
-		param_bind->setBlob(parameterIndex, NULL, false);
+	{
+		MySQL_ParamBind::Blob_t dummy;
+    param_bind->setBlob(parameterIndex, dummy, false);
 		param_bind->unset(parameterIndex);
 	}
 
@@ -600,7 +786,7 @@ MySQL_Prepared_Statement::setUInt(unsigned int parameterIndex, uint32_t value)
 	BufferSizePair p = allocate_buffer_for_type(t);
 
 	param_bind->set(parameterIndex);
-	MYSQL_BIND * param = &param_bind->get()[parameterIndex];
+	MYSQL_BIND * param = &param_bind->getBindObject()[parameterIndex];
 
 	param->buffer_type	= t;
 	delete [] static_cast<char *>(param->buffer);
@@ -629,8 +815,9 @@ MySQL_Prepared_Statement::setInt64(unsigned int parameterIndex, int64_t value)
 	}
 	--parameterIndex; /* DBC counts from 1 */
 
-	if (param_bind->getBlobObject(parameterIndex)) {
-		param_bind->setBlob(parameterIndex, NULL, false);
+	{
+		MySQL_ParamBind::Blob_t dummy;
+		param_bind->setBlob(parameterIndex, dummy, false);
 		param_bind->unset(parameterIndex);
 	}
 
@@ -639,7 +826,7 @@ MySQL_Prepared_Statement::setInt64(unsigned int parameterIndex, int64_t value)
 	BufferSizePair p = allocate_buffer_for_type(t);
 
 	param_bind->set(parameterIndex);
-	MYSQL_BIND * param = &param_bind->get()[parameterIndex];
+	MYSQL_BIND * param = &param_bind->getBindObject()[parameterIndex];
 
 	param->buffer_type	= t;
 	delete [] static_cast<char *>(param->buffer);
@@ -667,8 +854,9 @@ MySQL_Prepared_Statement::setUInt64(unsigned int parameterIndex, uint64_t value)
 	}
 	--parameterIndex; /* DBC counts from 1 */
 
-	if (param_bind->getBlobObject(parameterIndex)) {
-		param_bind->setBlob(parameterIndex, NULL, false);
+	{
+		MySQL_ParamBind::Blob_t dummy;
+		param_bind->setBlob(parameterIndex, dummy, false);
 		param_bind->unset(parameterIndex);
 	}
 
@@ -678,7 +866,7 @@ MySQL_Prepared_Statement::setUInt64(unsigned int parameterIndex, uint64_t value)
 	BufferSizePair p = allocate_buffer_for_type(t);
 
 	param_bind->set(parameterIndex);
-	MYSQL_BIND * param = &param_bind->get()[parameterIndex];
+	MYSQL_BIND * param = &param_bind->getBindObject()[parameterIndex];
 
 	param->buffer_type	= t;
 	delete [] static_cast<char *>(param->buffer);
@@ -708,8 +896,9 @@ MySQL_Prepared_Statement::setNull(unsigned int parameterIndex, int /* sqlType */
 	}
 	--parameterIndex; /* DBC counts from 1 */
 
-	if (param_bind->getBlobObject(parameterIndex)) {
-		param_bind->setBlob(parameterIndex, NULL, false);
+	{
+		MySQL_ParamBind::Blob_t dummy;
+		param_bind->setBlob(parameterIndex, dummy, false);
 		param_bind->unset(parameterIndex);
 	}
 
@@ -718,7 +907,7 @@ MySQL_Prepared_Statement::setNull(unsigned int parameterIndex, int /* sqlType */
 	BufferSizePair p = allocate_buffer_for_type(t);
 
 	param_bind->set(parameterIndex);
-	MYSQL_BIND * param = &param_bind->get()[parameterIndex];
+	MYSQL_BIND * param = &param_bind->getBindObject()[parameterIndex];
 
 	param->buffer_type	= t;
 	delete [] static_cast<char *>(param->buffer);
@@ -731,7 +920,7 @@ MySQL_Prepared_Statement::setNull(unsigned int parameterIndex, int /* sqlType */
 
 /* {{{ MySQL_Prepared_Statement::setString() -I- */
 void
-MySQL_Prepared_Statement::setString(unsigned int parameterIndex, const std::string& value)
+MySQL_Prepared_Statement::setString(unsigned int parameterIndex, const sql::SQLString& value)
 {
 	CPP_ENTER("MySQL_Prepared_Statement::setString");
 	CPP_INFO_FMT("this=%p", this);
@@ -743,20 +932,22 @@ MySQL_Prepared_Statement::setString(unsigned int parameterIndex, const std::stri
 		throw InvalidArgumentException("MySQL_Prepared_Statement::setString: invalid 'parameterIndex'");
 	}
 	if (value.length() > 256*1024) {
-		return setBlob_intern(parameterIndex, new std::istringstream(value), true);
+		MySQL_ParamBind::Blob_t dummy(const_cast<sql::SQLString*>(&value));
+		return param_bind->setBlob(--parameterIndex, dummy, false);
 	}
 
 	--parameterIndex; /* DBC counts from 1 */
 
-	if (param_bind->getBlobObject(parameterIndex)) {
-		param_bind->setBlob(parameterIndex, NULL, false);
+	{
+		MySQL_ParamBind::Blob_t dummy;
+		param_bind->setBlob(parameterIndex, dummy, false);
 		param_bind->unset(parameterIndex);
 	}
 
 	enum_field_types t = MYSQL_TYPE_STRING;
 
 	param_bind->set(parameterIndex);
-	MYSQL_BIND * param = &param_bind->get()[parameterIndex];
+	MYSQL_BIND * param = &param_bind->getBindObject()[parameterIndex];
 
 	delete [] static_cast<char *>(param->buffer);
 
@@ -771,6 +962,29 @@ MySQL_Prepared_Statement::setString(unsigned int parameterIndex, const std::stri
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_Statement::cancel() -U- */
+void
+MySQL_Prepared_Statement::cancel()
+{
+	CPP_ENTER("MySQL_Prepared_Statement::cancel");
+	CPP_INFO_FMT("this=%p", this);
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::cancel");
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_Statement::getFetchSize() -U- */
+size_t
+MySQL_Prepared_Statement::getFetchSize()
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::getFetchSize");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_Statement::getMetaData() -I- */
 sql::ResultSetMetaData *
 MySQL_Prepared_Statement::getMetaData()
@@ -802,33 +1016,49 @@ MySQL_Prepared_Statement::getResultSet()
 	CPP_ENTER("MySQL_Prepared_Statement::getResultSet");
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
-	if (mysql_more_results(mysql_stmt_conn(stmt))) {
-		mysql_next_result(mysql_stmt_conn(stmt));
-	}
 
 	my_bool	bool_tmp = 1;
-	mysql_stmt_attr_set(stmt, STMT_ATTR_UPDATE_MAX_LENGTH, &bool_tmp);
+	proxy->attr_set(STMT_ATTR_UPDATE_MAX_LENGTH, &bool_tmp);
 	sql::ResultSet::enum_type tmp_type;
 	if (resultset_type == sql::ResultSet::TYPE_SCROLL_INSENSITIVE) {
-		mysql_stmt_store_result(stmt);
+		if (proxy->store_result()) {
+			sql::mysql::util::throwSQLException(*proxy.get());
+		}
 		tmp_type = sql::ResultSet::TYPE_SCROLL_INSENSITIVE;
 	} else if (resultset_type == sql::ResultSet::TYPE_FORWARD_ONLY) {
 		tmp_type = sql::ResultSet::TYPE_FORWARD_ONLY;
 	} else {
-		throw SQLException("Invalid valude for result set type");
+		throw SQLException("Invalid value for result set type");
 	}
 
-	std::auto_ptr< MySQL_ResultBind > result_bind(new MySQL_ResultBind(stmt, logger));
+	sql::ResultSet * tmp = new MySQL_Prepared_ResultSet(proxy, result_bind, tmp_type, this, logger);
 
-	sql::ResultSet * tmp = new MySQL_Prepared_ResultSet(stmt, result_bind.get(), tmp_type, this, logger);
-	result_bind.release();
 	CPP_INFO_FMT("rset=%p", tmp);
-
 	return tmp;
 }
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_Statement::setFetchSize() -U- */
+void
+MySQL_Prepared_Statement::setFetchSize(size_t /* size */)
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::setFetchSize");
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_Statement::setQueryTimeout() -U- */
+void
+MySQL_Prepared_Statement::setQueryTimeout(unsigned int)
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::setQueryTimeout");
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_Statement::clearWarnings() -I- */
 void
 MySQL_Prepared_Statement::clearWarnings()
@@ -836,7 +1066,10 @@ MySQL_Prepared_Statement::clearWarnings()
 	CPP_ENTER("MySQL_Prepared_Statement::clearWarnings");
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
-	warnings.reset();
+	if (warnings)
+	{
+		warnings.reset();
+	}
 }
 /* }}} */
 
@@ -853,13 +1086,62 @@ MySQL_Prepared_Statement::close()
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_Statement::getMaxFieldSize() -U- */
+unsigned int
+MySQL_Prepared_Statement::getMaxFieldSize()
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::getMaxFieldSize");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_Statement::getMaxRows() -U- */
+uint64_t
+MySQL_Prepared_Statement::getMaxRows()
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::getMaxRows");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_Statement::getMoreResults() -U- */
 bool
 MySQL_Prepared_Statement::getMoreResults()
 {
+	CPP_ENTER("MySQL_Prepared_Statement::getMoreResults");
+	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
-	throw MethodNotImplementedException("MySQL_Prepared_Statement::getMoreResults");
-	return false; // fool compilers
+
+	if (proxy->more_results()) {
+
+		int next_result = proxy->stmt_next_result();
+
+		if (next_result == 0) {
+			return proxy->field_count() != 0;
+		} else if (next_result == -1) {
+			throw sql::SQLException("Impossible! more_results() said true, next_result says no more results");
+		} else {
+			CPP_ERR_FMT("Error during getMoreResults : %d:(%s) %s", proxy->errNo(), proxy->sqlstate().c_str(), proxy->error().c_str());
+			sql::mysql::util::throwSQLException(*proxy.get());
+		}
+	}
+	return false;
+
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_Statement::getQueryTimeout() -U- */
+unsigned int
+MySQL_Prepared_Statement::getQueryTimeout()
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::getQueryTimeout");
+	return 0; // fool compilers
 }
 /* }}} */
 
@@ -894,23 +1176,78 @@ MySQL_Prepared_Statement::getWarnings()
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
 
-	warnings.reset( loadMysqlWarnings( dynamic_cast<MySQL_Connection*>(connection) ) );
+	if (!warningsHaveBeenLoaded)
+	{
+		if (warningsCount)
+		warnings.reset( loadMysqlWarnings(connection, warningsCount) );
+		warningsHaveBeenLoaded= true;
+	}
 
 	return warnings.get();
 }
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_Statement::setCursorName() -U- */
+void
+MySQL_Prepared_Statement::setCursorName(const sql::SQLString &)
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::setCursorName");
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_Statement::setEscapeProcessing() -U- */
+void
+MySQL_Prepared_Statement::setEscapeProcessing(bool)
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::setEscapeProcessing");
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_Statement::setMaxFieldSize() -U- */
+void
+MySQL_Prepared_Statement::setMaxFieldSize(unsigned int)
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::setMaxFieldSize");
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_Statement::setMaxRows() -U- */
+void
+MySQL_Prepared_Statement::setMaxRows(unsigned int)
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::setMaxRows");
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_Statement::setResultSetConcurrency() -U- */
+void
+MySQL_Prepared_Statement::setResultSetConcurrency(int)
+{
+	checkClosed();
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::setResultSetConcurrency");
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_Statement::setResultSetType() -U- */
 sql::PreparedStatement *
-MySQL_Prepared_Statement::setResultSetType(sql::ResultSet::enum_type type)
+MySQL_Prepared_Statement::setResultSetType(sql::ResultSet::enum_type /* type */)
 {
 	checkClosed();
 #if WE_SUPPORT_USE_RESULT_WITH_PS
 		/* The connector is not ready for unbuffered as we need to refetch */
 	resultset_type = type;
 #else
-	throw MethodNotImplementedException("MySQL_Prepared_Statement::setResultSetType");	
+	throw MethodNotImplementedException("MySQL_Prepared_Statement::setResultSetType");
 #endif
 	return this;
 }
@@ -935,8 +1272,9 @@ void
 MySQL_Prepared_Statement::closeIntern()
 {
 	CPP_ENTER("MySQL_Prepared_Statement::closeIntern");
-	mysql_stmt_close(stmt);
+	proxy.reset();
 	clearParameters();
+	clearWarnings();
 
 	isClosed = true;
 }
