@@ -1,15 +1,30 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 
+
+
 #include <cppconn/exception.h>
 #include "mysql_util.h"
 #include "mysql_debug.h"
@@ -20,6 +35,8 @@
 
 #include <string.h>
 
+#include <boost/scoped_ptr.hpp>
+
 namespace sql
 {
 namespace mysql
@@ -144,7 +161,7 @@ void MySQL_ResultBind::bindResult()
 	len.reset(new unsigned long[num_fields]);
 	memset(len.get(), 0, sizeof(unsigned long) * num_fields);
 
-	std::auto_ptr< NativeAPI::NativeResultsetWrapper > resultMeta(proxy->result_metadata());
+	boost::scoped_ptr< NativeAPI::NativeResultsetWrapper > resultMeta(proxy->result_metadata());
 
 	for (unsigned int i = 0; i < num_fields; ++i) {
 		MYSQL_FIELD * field = resultMeta->fetch_field();
